#ifndef _SHFS_DCACHE_H_
#define _SHFS_DCACHE_H_

#define SHFS_DCACHE_LEN		5
#define SHFS_DCACHE_HASH	10
#define SHFS_DCACHE_TTL		30

#include <linux/slab.h>
#include <linux/kdev_t.h>

extern kmem_cache_t	*dir_hash_cache;
extern kmem_cache_t	*dir_list_cache;
extern kmem_cache_t	*dir_name_cache;

struct shfs_dir_entry {
	umode_t		mode;
	char        	*name;
	unsigned long	ino;
	uid_t		uid;
	gid_t		gid;
	off_t		size;
	kdev_t		rdev;
	nlink_t		nlink;
	unsigned long	blocksize;
	unsigned long	blocks;
	time_t		atime;
	int		changed;
};

struct shfs_dir_list_node {
	struct shfs_dir_list_node *prev, *next;
	struct shfs_dir_entry entry;
};

struct shfs_directory {
	struct shfs_dir_list_node *head;
	time_t                    time;
	char			  *name;
};

struct shfs_dir_hash_node {
	struct shfs_dir_hash_node *prev, *next;
	struct shfs_directory directory;
};

struct shfs_dir_cache {
	struct shfs_dir_hash_node *hash[SHFS_DCACHE_HASH];
	unsigned len[SHFS_DCACHE_HASH];
};

struct shfs_sb_info;

void shfs_dcache_init(struct shfs_sb_info*);
int shfs_dcache_empty(struct shfs_sb_info*);
int shfs_dcache_get(struct shfs_sb_info*, char*, struct shfs_directory**);
int shfs_dcache_revalidate(struct dentry *, struct shfs_sb_info*);
void shfs_dcache_invalidate(struct dentry*);
void shfs_dcache_update_fattr(struct dentry*, struct iattr*);

#endif	/* _SHFS_DCACHE_H_ */
